/*
 * $Id: findface.c,v 1.2 1995/03/13 19:23:46 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <faces.h>

static String fallbackResources[] = {
  "FindFace.geometry:  64x64",
  NULL
};

Display      *display;
Screen       *screen;
Window        window;
XtAppContext  app;
Widget        toplevel;

#if NeedFunctionPrototypes
void openDisplay(int *argc, char **argv)
#else
void openDisplay(argc, argv)
  int   *argc;
  char **argv;
#endif
{
  /* Create an application */
  toplevel = XtAppInitialize(&app, "FindFace", NULL, 0, argc, argv,
			     fallbackResources, NULL, 0);

  /* Realize the toplevel widget */
  XtRealizeWidget(toplevel);

  /* Now get the display, screen and window */
  display = XtDisplay(toplevel);
  screen  = XtScreen(toplevel);
  window  = XtWindow(toplevel);
}
  
#if NeedFunctionPrototypes
void main(int argc, char **argv)
#else
void main(argc, argv)
  int    argc;
  char **argv;
#endif
{
  Face *face;

  /* This function should be called before using any other faceslib
   * call */
  FaceInit();
  
  /* Open the display, this can be done before or after FaceInit but
   * should have been done before any other faceslib call */
  openDisplay(&argc, argv);

  /* See if a name is given, if not exit */
  if (argc < 2) {
    printf("Need a name to display!\n");
    exit(-1);
  }
  
  /* This is the real function. It will find a face belonging to the
   * email address, and return a Face structure. */
  face = FaceSearchDB(argv[1], NULL);

  /* Now display the image */
  XSetWindowBackgroundPixmap(display, window, face->image);

  /* Start the mainloop, just kill to exit. Not a nice program. */
  XtAppMainLoop(app);
}

