/*
 * $Id: nomail.c,v 1.3 1995/05/27 16:56:01 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#if SYSV
# include <string.h>
#else
# include <strings.h>
#endif
#include "faces.h"
#ifdef XPM
# include <X11/xpm.h>
#endif

/*
 * Default icons.
 */
#include "nomail.xpm"
#include "X11/bitmaps/mailempty"

Face *nomailFace;

#if NeedFunctionPrototypes
Face *loadNomail(void)
#else
Face *loadNomail()
#endif
{
  Face          *face;
  Pixmap         icon, image, shape;
#ifdef XPM
  XpmAttributes  attributes;

  attributes.valuemask = 0;
  XpmCreatePixmapFromData(display, window, nomail,
                          &image, &shape, &attributes);
  if (!facesOptions.UseColorIcon)
    icon = XCreateBitmapFromData(display, window, mailempty_bits,
				 mailempty_width, mailempty_height);
  else
    icon = image;
#else
  icon = XCreateBitmapFromData(display, window, mailempty_bits,
			       mailempty_width, mailempty_height);
  if (DefaultDepthOfScreen(screen) > 1) {
    image = XCreatePixmap(display, window,
			  mailempty_width, mailempty_height,
			  DefaultDepthOfScreen(screen));
    XCopyPlane(display, icon, image, gc,
	       0, 0, mailempty_width, mailempty_height, 0, 0, 1);
  } else
    image = icon;
  shape = None;
#endif

  face          = XtNew(Face);
  face->name    = XtNewString("No Mail");
#ifdef XPM
  face->width  = attributes.width;
  face->height = attributes.height;
  if (facesOptions.UseColorIcon) {
    face->icon_width  = attributes.width;
    face->icon_height = attributes.height;
  } else {
    face->icon_width  = mailempty_width;
    face->icon_height = mailempty_height;
  }
    
#else
  face->width       = mailempty_width;
  face->height      = mailempty_height;
  face->icon_width  = mailempty_width;
  face->icon_height = mailempty_height;
#endif
  face->shape   = shape;
  face->image   = image;
  face->icon    = icon;
  face->counter = 1;
  face->unknown = True;
  face->next    = NULL;
  face->prev    = NULL;

  return face;
}
