.\" @(#)$Id: libfaces.man,v 1.4 1995/05/30 20:52:17 kooper Exp $
.\" tbl libfaces.man | nroff -man | col -b
.TH LIBFACES 3 "March 16 1995"
.SH NAME
libfaces \- finds a face with an email address.
.SH SYNOPSIS
.B #include <faces.h>
.PP
.SM
.B extern Face *firstFace;
.PP
.B void
.br
.B FaceInit (void)
.PP
.B void
.br
.B FaceClose (void)
.PP
.B void
.br
.B FaceRemoveOne (faceToBeRemoved)
.br
.B Face *faceToBeRemoved;
.PP
.B void
.br
.B FaceRemoveAll (void)
.PP
.B void
.br
.B FaceDelete (faceToDelete)
.br
.B Face *faceToDelete;
.PP
.B void
.br
.B FaceInitDB (void)
.PP
.B void
.br
.B FaceOpenDB (void)
.PP
.B void
.br
.B FaceCloseDB (void)
.PP
.B Face *
.br
.B FaceSearchDB (emailAddress, XFaceData)
.br
.B char *emailAddress, *XFaceData;
.SH DESCRIPTION
The faces library is intended to make it easier to use faces in your
program. After initializing the library can be called and will return
a structure containing the found face. It search the email-address for
a fitting face. It searches in the following manner:
.PP
.RS 12
kooper@cc.gatech.edu
.br
kooper@gatech.edu
.br
kooper@edu
.br
kooper@MISC
.br
unknown@cc.gatech.edu
.br
unknown@gatech.edu
.br
unknown@edu
.br
unknown@MISC
.br
  unknown face
.RE
There are three different algorithms for the library to search for a
face. The default algorithm is
.B searchKnown.
.SS FUNCTIONS
.TP
.B void FaceInit (void)
This function is to initialize some global variables used in the
faces library and should be called before any other function in the
faces library is used. If this is not done than the results are
unpredictable.
.TP
.B void FaceClose (void)
This function will clean up all the memory allocated and remove the
pixmaps from the display.
.TP
.B void FaceRemoveOne (faceToBeRemoved)
This function removes a face from the list of faces, frees up the
memory and removes the pixmap from the display.
.TP
.B void FaceRemoveAll (void)
This function does the same as the above function but will remove all
the faces in the list of faces.
.TP
.B void FaceDelete (faceToDelete)
This function will remove one instance of the face and if this was the
last instance remove it, using FaceRemoveOne.
.TP
.B void FaceInitDB (void)
This function will build a list of all the databases that can be
found. If you add a new database, this function has to be called to
add it to the list of known databases.
It will also try to open the mailcap file and load the entries, this
will be used to display faces that are not of the form image/xbm,
image/xpm or image/gif. The mailcap is searched in the following
places:
.RS 12
$MAILCAP
.br
$HOME/.mailcap
.br
/usr/lib/mailcap
.br
/usr/local/lib/mailcap
.RE
.TP
.B void FaceOpenDB (void)
This function is for speed. It will open all the databases and keep
them open till
.B FaceCloseDB
is called. This means that no modifications can be made to the
databases until
.B FaceCloseDB
is called.
.TP
.B void FaceCloseDB (void)
Will close all the databases that are open.
.TP
.B Face *FaceSearchDB (emailAddress, XFaceData)
Will look for the variable
.I facesOptions.SearchFunction
and call the appropiate function.
.TP
.B Breadth search
This function will open the databases if necessary, and close them at
the end again. It will then search the databases until a face of the
form gif, xpm or xbm is found. Any other face that is found before
that will be displayed using the mailcap.
.TP
.B Depth search
This function implements the same algorithm as in
.I faces
and
.I xfaces.
It will open a databases if necessary, search that database for all
queries, so from kooper@cc.gatech.edu to unknown@MISC, and close them
at the end again. It will then search another databases until a face
of the form gif, xpm or xbm is found. Any other face that is found
before that will be displayed using the mailcap.
.TP
.B Known search
This function implements an alogorithm that is a cross between
.B FaceDepthSearchDB
and
.B FaceBreadthSearchDB.
It starts looking in a database for all known queries, so from
kooper@cc.gatech.edu to kooper@MISC, then moves on to the next
database, and the starts looking for all unknown queries, thus from
unknown@cc.gatech.edu to unknown@MISC. It will continue until a face
of the form gif, xpm or xbm is found. Any other face that is found
before that will be displayed using the mailcap.
.SS STRUCTURES
The faces program uses two structures that are important for the
user. One structure which contains the face that is returned and one
structure that holds the options for the faces program.
.br
typedef struct Face {
  char         *name;
  struct Face  *iconPtr;
  Pixmap        icon, image, shape;
  int           width, height;
  int           counter;
  int           unknown;
  struct Face  *next, *prev;
.br
} Face;
.TP
.I Name
will hold the email address to which this face belongs.
.TP
.I icon, image, shape
will contain pixmaps of the face, these can be used in your program.
.TP
.I counter
counts the number of times the face is requested.
.TP
.I unknown
Will be true if this face is the unknown face, when no face could be
matched to the email address.
.PP
The other structure will hold all the options you can set. To change
an option just change faceOptions.<whatever>. If for instance you want
to change the 
.I ImageOrder
to xbm, you can use in your program:
.br
.B faceOptions.ImageOrder = XtNewString("xbm");
.br
typedef struct FacesOptions {
  Pixel         Background;
  Pixel         Foreground;
  char         *DatabaseOrder;
  int           Debug;
  char         *ImageOrder;
  char         *PathNames;
  int           SearchFunction;
  char         *TmpFile;
  Boolean       UseMailcap;
  Boolean       UseColorIcon;
  Boolean       XfaceFirst;
.br
} FacesOptions;
.TP
.I Background, Forground
are the back and foreground colors used when loading a bitmap.
.TP
.I DatabaseOrder
is a list of databases to be searched. The library will search for
these databases in all parts of
.I PathNames.
If the database was not found in the directory it will try to append
.I \.gdbm
to the name.
.TP
.I Debug
will print some debug info, useful when testing your program, annoying
otherwise.
.TP
.I ImageOrder
is the order in which the faces should be searched, this list is colon
separated. This is by default gif:xpm:xbm. If for example an address
has both a xpm and xbm image then the xpm will be used as the face, if
you don't want xpm images, and support for them is compiled in the
library, you can set this to xbm. After checking for an xbm or xpm
image, and ImageOrder contains a '*', the rest is displayed using the
mailcap.
.TP
.I SearchFunction
is the algorithm that should be used to find a face. It can be one of
the following values searchDepth (1), searchBreadth (2) and
searchKnown (3).
.TP
.I PathNames
will contain a colon separated lists of places to search for
databases. It will search each path of this list for all the databases
in
.I DatabaseOrder
before moving on to the next path in this list.
.TP
.I TmpFile
is the place and first part of the filename where to store things that
have to be displayed using external programs. This is by default
.I /tmp/libfaces.
The files whill be called <faceOptions.TmpFile>_[a-z][a-z]
.TP
.I UseMailcap
is a flag that tells the library whether or not it should use the
mailcap.  If this is set to True, the default, the library will
display unknown entries using external programs. When set to False the
library will skip the entry.
.TP
.I UseColorIcon
if this is True, default for the SGI's, it will use color icons. If
this if False it will search for an B&W icon.
.TP
.I XfaceFirst
If this flag is set True and
.B Face*SearchDB
is called with an xface, then the library will use this before
searching the databases. If set to False, the default, the library
will use the xface when starting to look for unknown faces.
.SS GLOBAL VARIABLES
There are three variables that the faces library expects to be set in
the calling program.
.br
.B Display *display;
It the display on which the images will be displayed.
.br
.B Screen *screen;
The screen connected to the display
.br
.B Window window;
The window in which the pixmaps should be loaded.
.PP
For example the following piece of code, using the Xt library will set
all these variables:
.RS 4
Display *display;
.br
Screen *screen;
.br
Window window;

void openDisplay(int *argc, char **argv) {
  Widget toplevel;

  /* Create an application */
  toplevel = XtAppInitialize(&app, "Faces",
	options, XtNumber(options), argc, argv,
.br
	NULL, NULL, 0);

  /* Realize the toplevel widget */
  XtRealizeWidget(toplevel);

  /* Now get the display, screen and window */
  display = XtDisplay(toplevel);
  screen  = XtScreen(toplevel);
  window  = XtWindow(toplevel);
.br
}
.RE
The following variable is a list of all faces that were requested to
with the FaceSearchDB function.
.B Face *firstFace;
.SH EXAMPLE
void main() {
  /* This function should be called before using any other
   * faceslib call */
  FaceInit();

  /* Open the display, this can be done before or after
   * FaceInit but should have been done before any other
   * faceslib call */
  openDisplay();

  /* This is not really necessary. It is done automaticly on
   * the first search or openDB call. */
  FaceInitDB();

  /* Some large loop. */
  while(loop) {

    /* Open the database, this is done for speed. If only
     * one face is searched at the time, you don't need this
     * call. */
    FaceOpenDB();

    /* This is the real function. It will find a face
     * belonging to the email address, and return a Face
     * structure. */
    face = FaceSearchDB("kooper@cc.gatech.edu", NULL);

    /* You don't need to close if you didn't open the
     * databases. */
    FaceCloseDB();
  }

  /* This will free memory and pixmaps allocated by the
   * program. */
  FaceClose();

  /* Close the display and prepare for shutdown. */
  closeDisplay();
.br
}
.SH ENVIRONMENT VARIABLES
.TP
.B FACEPATH
is a colon separated list of paths to search the databases. The
variable will be prepended to the list kept in 
.I faceOptions.PathNames.
.TP
.B FACEDB
is a colon separated list of databases to search. The variable will
again be prepended to the list kept in
.I faceOptions.DatabaseOrder
.TP
.B MAILCAP
is a pointer to where your personal mailcap file is kept. This
variable can be set to use an experimental mailcap file for the
library.
.SH "SEE ALSO"
faces(1), facesdb(1), compface(1), uncompface(1), compface(3),
mail(1), sendmail(8)
.SH BUGS
There are probably bugs somewhere, and some things can be done
better. If you find anything, please send me email.
.SH AUTHOR
Rob Kooper (kooper@cc.gatech.edu)
.SH ACKNOWLEDGEMENTS
Thanks go to Steve Kinzler (kinzler@indiana.edu) for helping me with
the database format, James Ashton for writing the compface library,
Christopher B. Liebman (liebman@zod.clark.net) for the
.I xface
program,  Rich Burridge for the
.I faces
program, and David Koblas (koblas@netcom.com) for the code to load a
GIF file.
