/*
 * $Id: load_gif.c,v 1.2 1995/05/30 20:52:17 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* +-------------------------------------------------------------------+ */
/* | Copyright 1990, 1991, 1993, David Koblas.  (koblas@netcom.com)    | */
/* |   Permission to use, copy, modify, and distribute this software   | */
/* |   and its documentation for any purpose and without fee is hereby | */
/* |   granted, provided that the above copyright notice appear in all | */
/* |   copies and that both that copyright notice and this permission  | */
/* |   notice appear in supporting documentation.  This software is    | */
/* |   provided "as is" without express or implied warranty.           | */
/* +-------------------------------------------------------------------+ */


#ifdef XPM

#include <stdio.h>
#if SYSV
# include <string.h>
#else
# include <strings.h>
#endif
#include <sys/stat.h>
#include "facesP.h"
#include <X11/xpm.h>

#define MAXCOLORMAPSIZE 256

#define TRUE            1
#define FALSE           0

#define CM_RED          0
#define CM_GREEN        1
#define CM_BLUE         2

#define MAX_LWZ_BITS    12

#define INTERLACE       0x40
#define LOCALCOLORMAP   0x80

#define BitSet(byte, bit)       (((byte) & (bit)) == (bit))
#define LM_to_uint(a,b)         (((b)<<8)|(a))

struct {
       unsigned int    Width;
       unsigned int    Height;
       unsigned char   ColorMap[3][MAXCOLORMAPSIZE];
       unsigned int    BitPixel;
       unsigned int    ColorResolution;
       unsigned int    Background;
       unsigned int    AspectRatio;
       int             Grayscale;
} GifScreen;

struct {
       int     transparent;
       int     delayTime;
       int     inputFlag;
       int     disposal;
} Gif89 = { -1, -1, -1, 0 };


/*
 * Local prototypes.
 */
#if NeedFunctionPrototypes
Boolean ReadOK      (char **, char *, int);
void    ReadColorMap(char **, int, unsigned char [3][MAXCOLORMAPSIZE]);
int     DoExtension (char **, int);
int     GetDataBlock(char **, unsigned char *);
int     GetCode     (char **, int, int);
int     LWZReadByte (char **, int, int);
char   *ReadImage   (char **, int, int, unsigned char [3][MAXCOLORMAPSIZE],
		     int, int);
void    ReadGIF     (char **);
#else
Boolean ReadOK      ();
void    ReadColorMap();
int     DoExtension ();
int     GetDataBlock();
int     GetCode     ();
int     LWZReadByte ();
char   *ReadImage   ();
void    ReadGIF     ();
#endif

#if NeedFunctionPrototypes
void face_loadGif(Face **facePtr, char *data, int length)
#else
void face_loadGif(facePtr, data, length)
  Face **facePtr;
  char  *data;
  int    length;
#endif
{
  FILE            *fp;
  char            *buffer;
  struct stat      buf;

  /*
   * If this function gets called with face information we are just
   * interested in a xbm icon. So return without loading this one.
   */
  if (*facePtr != NULL)
    return;

  /*
   * Load the gif file in a buffer.
   */
  if (length > 0) {
    buffer = XtMalloc(length);
    memcpy(buffer, data, length);
  } else {
    /*
     * Get the size of the image.
     */
    if (stat(data, &buf) < 0) {
      WARNING(("Could not find image %s.", data));
      return;
    }
    length = buf.st_size;
    buffer = XtMalloc(length);

    /*
     * Load the image in a buffer.
     */
    if ((fp = fopen(data, "rb")) == NULL) {
      WARNING(("Could not open image %s.", data));
      return;
    }
    if (fread(buffer, 1, length, fp) != length) {
      fclose(fp);
      WARNING(("Could not read the image (%s).\n", data));
      return;
    }
    fclose(fp);
  }


  /*
   * Convert the gif buffer to xpm and load it using face_loadXpm
   */
  ReadGIF(&buffer);
  face_loadXpm(facePtr, buffer, strlen(buffer));

  XtFree(buffer);
}


/* +-------------------------------------------------------------------+ */
/* | Copyright 1990, 1991, 1993, David Koblas.  (koblas@netcom.com)    | */
/* |   Permission to use, copy, modify, and distribute this software   | */
/* |   and its documentation for any purpose and without fee is hereby | */
/* |   granted, provided that the above copyright notice appear in all | */
/* |   copies and that both that copyright notice and this permission  | */
/* |   notice appear in supporting documentation.  This software is    | */
/* |   provided "as is" without express or implied warranty.           | */
/* +-------------------------------------------------------------------+ */

#if NeedFunctionPrototypes
void ReadGIF(char **file)
#else
void ReadGIF(file)
  char **file;
#endif
{
  char          *orig, *xpm;
  unsigned char  buf[16];
  unsigned char  c;
  unsigned char  localColorMap[3][MAXCOLORMAPSIZE];
  int            grayScale;
  int            useGlobalColormap;
  int            bitPixel;
  char           version[4];

  orig = *file;

  if (! ReadOK(file, buf, 6))
    ERROR(("error reading magic number"));

  if (strncmp((char *)buf, "GIF", 3) != 0)
    ERROR(("not a GIF file"));

  strncpy(version, (char *)buf + 3, 3);
  version[3] = '\0';

  if ((strcmp(version, "87a") != 0) && (strcmp(version, "89a") != 0))
    ERROR(("bad version number, not '87a' or '89a'"));

  if (! ReadOK(file, buf, 7))
    ERROR(("failed to read screen descriptor"));

  GifScreen.Width           = LM_to_uint(buf[0],buf[1]);
  GifScreen.Height          = LM_to_uint(buf[2],buf[3]);
  GifScreen.BitPixel        = 2<<(buf[4]&0x07);
  GifScreen.ColorResolution = (((buf[4]&0x70)>>3)+1);
  GifScreen.Background      = buf[5];
  GifScreen.AspectRatio     = buf[6];

  if (BitSet(buf[4], LOCALCOLORMAP))      /* Global Colormap */
    ReadColorMap(file, GifScreen.BitPixel, GifScreen.ColorMap);

  if (GifScreen.AspectRatio != 0 && GifScreen.AspectRatio != 49)
    WARNING(("warning - non-square pixels."));

  for (;;) {
    if (! ReadOK(file, &c, 1))
      ERROR(("EOF / read error on image data"));

    if (c == ';') {         /* GIF terminator */
      return;
    }

    if (c == '!') {         /* Extension */
      if (! ReadOK(file, &c, 1))
	ERROR(("OF / read error on extention function code"));
      DoExtension(file, c);
      continue;
    }

    if (c != ',') {         /* Not a valid start character */
      WARNING(("bogus character 0x%02x, ignoring", (int) c));
      continue;
    }

    if (! ReadOK(file, buf, 9))
      ERROR(("couldn't read left/top/width/height"));

    useGlobalColormap = ! BitSet(buf[8], LOCALCOLORMAP);

    bitPixel = 1<<((buf[8]&0x07)+1);

    if (! useGlobalColormap) {
      ReadColorMap(file, bitPixel, localColorMap);
      xpm = ReadImage(file, LM_to_uint(buf[4],buf[5]),
		      LM_to_uint(buf[6],buf[7]), 
		      localColorMap, bitPixel,
		      BitSet(buf[8], INTERLACE));
    } else {
      xpm = ReadImage(file, LM_to_uint(buf[4],buf[5]),
		      LM_to_uint(buf[6],buf[7]), 
		      GifScreen.ColorMap, GifScreen.BitPixel,
		      BitSet(buf[8], INTERLACE));
    }

  /*
   * I only want the first image. So after loading it, I can return.
   */
  XtFree(orig);
  *file = xpm;
  return;

  }
}

#if NeedFunctionPrototypes
void ReadColorMap(char **file, int number,
		  unsigned char buffer[3][MAXCOLORMAPSIZE])
#else
void ReadColorMap(file, number, buffer)
  char          **file;
  int             number;
  unsigned char   buffer[3][MAXCOLORMAPSIZE];
#endif
{
  int             i;
  unsigned char   rgb[3];

  for (i = 0; i < number; ++i) {
    if (! ReadOK(file, rgb, sizeof(rgb)))
      ERROR(("bad colormap"));
	    
    buffer[CM_RED][i]   = rgb[0] ;
    buffer[CM_GREEN][i] = rgb[1] ;
    buffer[CM_BLUE][i]  = rgb[2] ;
  }
}

#if NeedFunctionPrototypes
int DoExtension(char **file, int label)
#else
int DoExtension(file, label)
  char **file;
  int    label;
#endif
{
  static char     buf[256];
  char            *str;

  switch (label) {
  case 0x01:              /* Plain Text Extension */
    str = "Plain Text Extension";
    break;
  case 0xff:              /* Application Extension */
    str = "Application Extension";
    break;
  case 0xfe:              /* Comment Extension */
    str = "Comment Extension";
    while (GetDataBlock(file, (unsigned char*) buf) != 0) {
      if (facesOptions.Debug)
	printf("gif comment: %s", buf);
    }
    return FALSE;
  case 0xf9:              /* Graphic Control Extension */
    str = "Graphic Control Extension";
    (void) GetDataBlock(file, (unsigned char*) buf);
    Gif89.disposal    = (buf[0] >> 2) & 0x7;
    Gif89.inputFlag   = (buf[0] >> 1) & 0x1;
    Gif89.delayTime   = LM_to_uint(buf[1],buf[2]);
    if ((buf[0] & 0x1) != 0)
      Gif89.transparent = buf[3];
    
    while (GetDataBlock(file, (unsigned char*) buf) != 0)
                       ;
    return FALSE;
  default:
    str = buf;
    sprintf(buf, "UNKNOWN (0x%02x)", label);
    break;
  }

  WARNING(("got a '%s' extension", str));

  while (GetDataBlock(file, (unsigned char*) buf) != 0)
    ;

  return FALSE;
}

int    ZeroDataBlock = FALSE;

#if NeedFunctionPrototypes
int GetDataBlock(char **file, unsigned char *buf)
#else
int GetDataBlock(file, buf)
  char          **file;
  unsigned char  *buf;
#endif
{
  unsigned char   count;

  if (! ReadOK(file, &count, 1)) {
    WARNING(("error in getting DataBlock size"));
    return -1;
  }

  ZeroDataBlock = count == 0;

  if ((count != 0) && (! ReadOK(file, buf, count))) {
    WARNING(("error in reading DataBlock"));
    return -1;
  }

  return count;
}

#if NeedFunctionPrototypes
int GetCode(char **file, int code_size, int flag)
#else
int GetCode(file, code_size, flag)
  char **file;
  int    code_size;
  int    flag;
#endif
{
  static unsigned char    buf[280];
  static int              curbit, lastbit, done, last_byte;
  int                     i, j, ret;
  unsigned char           count;
  
  if (flag) {
    curbit = 0;
    lastbit = 0;
    done = FALSE;
    return 0;
  }

  if ( (curbit+code_size) >= lastbit) {
    if (done) {
      if (curbit >= lastbit)
	ERROR(("ran off the end of my bits"));
      return -1;
    }
    buf[0] = buf[last_byte-2];
    buf[1] = buf[last_byte-1];

    if ((count = GetDataBlock(file, &buf[2])) == 0)
      done = TRUE;

    last_byte = 2 + count;
    curbit = (curbit - lastbit) + 16;
    lastbit = (2+count)*8 ;
  }

  ret = 0;
  for (i = curbit, j = 0; j < code_size; ++i, ++j)
    ret |= ((buf[ i / 8 ] & (1 << (i % 8))) != 0) << j;

  curbit += code_size;

  return ret;
}

#if NeedFunctionPrototypes
int LWZReadByte(char **file, int flag, int input_code_size)
#else
int LWZReadByte(file, flag, input_code_size)
  char **file;
  int    flag;
  int    input_code_size;
#endif
{
  static int      fresh = FALSE;
  int             code, incode;
  static int      code_size, set_code_size;
  static int      max_code, max_code_size;
  static int      firstcode, oldcode;
  static int      clear_code, end_code;
  static int      table[2][(1<< MAX_LWZ_BITS)];
  static int      stack[(1<<(MAX_LWZ_BITS))*2], *sp;
  register int    i;

  if (flag) {
    set_code_size = input_code_size;
    code_size = set_code_size+1;
    clear_code = 1 << set_code_size ;
    end_code = clear_code + 1;
    max_code_size = 2*clear_code;
    max_code = clear_code+2;
    
    GetCode(file, 0, TRUE);
    
    fresh = TRUE;
    
    for (i = 0; i < clear_code; ++i) {
      table[0][i] = 0;
      table[1][i] = i;
    }
    for (; i < (1<<MAX_LWZ_BITS); ++i)
      table[0][i] = table[1][0] = 0;
    
    sp = stack;
    
    return 0;
  } else if (fresh) {
    fresh = FALSE;
    do {
      firstcode = oldcode = GetCode(file, code_size, FALSE);
    } while (firstcode == clear_code);
    return firstcode;
  }

  if (sp > stack)
    return *--sp;
  
  while ((code = GetCode(file, code_size, FALSE)) >= 0) {
    if (code == clear_code) {
      for (i = 0; i < clear_code; ++i) {
	table[0][i] = 0;
	table[1][i] = i;
      }
      for (; i < (1<<MAX_LWZ_BITS); ++i)
	table[0][i] = table[1][i] = 0;
      code_size = set_code_size+1;
      max_code_size = 2*clear_code;
      max_code = clear_code+2;
      sp = stack;
      firstcode = oldcode = GetCode(file, code_size, FALSE);
      return firstcode;
    } else if (code == end_code) {
      int             count;
      unsigned char   buf[260];
      
      if (ZeroDataBlock)
	return -2;
      
      while ((count = GetDataBlock(file, buf)) > 0)
	;
      
      if (count != 0)
	WARNING(("missing EOD in data stream (common occurence)"));
      return -2;
    }

    incode = code;

    if (code >= max_code) {
      *sp++ = firstcode;
      code = oldcode;
    }

    while (code >= clear_code) {
      *sp++ = table[1][code];
      if (code == table[0][code])
	ERROR(("circular table entry BIG ERROR"));
      code = table[0][code];
    }

    *sp++ = firstcode = table[1][code];
    
    if ((code = max_code) <(1<<MAX_LWZ_BITS)) {
      table[0][code] = oldcode;
      table[1][code] = firstcode;
      ++max_code;
      if ((max_code >= max_code_size) && (max_code_size < (1<<MAX_LWZ_BITS))) {
	max_code_size *= 2;
	++code_size;
      }
    }

    oldcode = incode;
    
    if (sp > stack)
      return *--sp;
  }
  return code;
}

#if NeedFunctionPrototypes
char *ReadImage(char **file, int len, int height,
		unsigned char cmap[3][MAXCOLORMAPSIZE],
		int number, int interlace)
#else
char *ReadImage(file, len, height, cmap, number, interlace)
  char          **file;
  int             len, height;
  unsigned char   cmap[3][MAXCOLORMAPSIZE];
  int             interlace, number;
#endif
{
  char            str[256];
  char            col[3];
  int            *image;
  unsigned char   c;      
  int             v;
  int             xpos = 0, ypos = 0, pass = 0;
  char           *xpm;
  Boolean         stop;

  /*
   *  Initialize the Compression routines
   */
  if (! ReadOK(file, &c, 1))
    ERROR(("EOF / read error on image data"));

  if (LWZReadByte(file, TRUE, c) < 0)
    ERROR(("error reading image"));

  /*
   * Calculate the space the xpm will take. (
   *        112 bytes for the xpm headers.
   *         16 bytes for each color.
   *  (len*2+4) bytes for each line in the image
   */
  sprintf(str, "\"%d %d %d 2\",\n", len, height, number);
  v = strlen(str) + 112 + 16*number + height*(len*2+4) + 1;
  if (Gif89.transparent != -1)
    v += 4;

  /*
   * Allocate space for the xpm.
   */
  image = (int *)XtMalloc(len * height * sizeof(int));
  xpm   = XtMalloc(v);

  /*
   * Add the standard xpm headers.
   */
  strcpy(xpm, "/* XPM */\n");
  strcat(xpm, "static char * face_xpm[] = {\n");
  strcat(xpm, "/* width height ncolors chars_per_pixel */\n");
  strcat(xpm, str);
  
  /*
   * Add the colormap to the xpm file.
   */
  strcat(xpm, "/* colors */\n");
  for (v = 0; v < number; v++) {
    col[0] = 'a' + (v/32);
    col[1] = 'a' + (v%32);
    col[2] = 0;
    if (v == Gif89.transparent)
      sprintf(str, "\"%s s None c None\",\n", col);
    else
      sprintf(str, "\"%s c #%02x%02x%02x\",\n", col,
	      cmap[CM_RED][v], cmap[CM_GREEN][v], cmap[CM_BLUE][v]);
    strcat(xpm, str);
  }

  /*
   * Read the rest of the image in the xpm.
   */
  if (facesOptions.Debug)
    printf("reading %d by %d%s GIF image", len, height,
	   interlace ? " interlaced" : "");

  stop = False;
  while (!stop && ((v = LWZReadByte(file, FALSE, c)) >= 0 )) {
    image[(ypos*len)+xpos] = v;

    ++xpos;
    if (xpos == len) {
      xpos = 0;
      if (interlace) {
	switch (pass) {
	case 0:
	case 1:
	  ypos += 8; break;
	case 2:
	  ypos += 4; break;
	case 3:
	  ypos += 2; break;
	}
	
	if (ypos >= height) {
	  ++pass;
	  switch (pass) {
	  case 1:
	    ypos = 4; break;
	  case 2:
	    ypos = 2; break;
	  case 3:
	    ypos = 1; break;
	  default:
	    stop = True; break;
	  }
	}
      } else {
	++ypos;
      }
    }
    if (ypos >= height)
      stop = True;
  }

  /*
   * Convert the image to xpm
   */
  strcat(xpm, "/* pixels */\n");
  for (ypos = 0; ypos < height; ypos++) {
    strcat(xpm, "\"");
    for (xpos = 0; xpos < len; xpos++) {
      col[0] = 'a' + (image[(ypos*len)+xpos]/32);
      col[1] = 'a' + (image[(ypos*len)+xpos]%32);
      col[2] = 0;
      strcat(xpm, col);
    }
    strcat(xpm, "\",\n");
  }
  strcat(xpm, "};\n");

  /*
   * Clean up space.
   */
  XtFree((char*)image);
  return xpm;
}


#if NeedFunctionPrototypes
Boolean ReadOK(char **file, char *buffer, int len)
#else
Boolean ReadOK(file, buffer, len)
  char **file, *buffer;
  int    len;
#endif
{
  memcpy(buffer, *file, len);

  (*file) += len;

  if (buffer != NULL)
    return True;
  return False;
}

#endif /* XPM */
