/*
 * $Id: load_xbm.c,v 1.8 1995/05/30 20:52:17 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#if SYSV
# include <string.h>
#else
# include <strings.h>
#endif
#include "facesP.h"

/*
 * Local prototypes.
 */
#if NeedFunctionPrototypes
int face_dataOfXbm(char *, int , Pixmap *, unsigned int *, unsigned int *);
#else
int face_dataOfXbm();
#endif

#if NeedFunctionPrototypes
void face_loadXbm(Face **facePtr, char *data, int length)
#else
void face_loadXbm(facePtr, data, length)
  Face **facePtr;
  char  *data;
  int    length;
#endif
{
  Pixmap        icon, image;
  unsigned int  width, height;
  int           hot_x, hot_y, xbmError;
  Face         *face;

  face = *facePtr;

  if (length > 0)
    xbmError = face_dataOfXbm(data, length, &icon, &width, &height);
  else
    xbmError = XReadBitmapFile(display, window, data, &width, &height,
			       &icon, &hot_x, &hot_y);

  switch (xbmError) {
  case BitmapSuccess :
    if (FacesPrivate.depth > 1) {
      image = XCreatePixmap(display, window, width, height,
			    FacesPrivate.depth);
      XCopyPlane(display, icon, image, FacesPrivate.gc,
		 0, 0, width, height, 0, 0, 1);
    } else
      image = icon;
    
    if (face == NULL) {
      face          = XtNew(Face);
      *facePtr      = face;
      face->name    = NULL;
      face->iconPtr = NULL;
      face->width   = width;
      face->height  = height;
      face->image   = image;
      face->shape   = None;
      face->counter = 1;
      face->unknown = False;
      face->next    = NULL;
      face->prev    = NULL;
    }
    face->icon_width  = width;
    face->icon_height = height;
    face->icon        = icon;
    break;

  case BitmapOpenFailed  :
    if (length > 0) {
      WARNING (("Could not open the data???"));
    } else {
      WARNING (("Could not open file %s", data));
    }
    break;
    
  case BitmapFileInvalid :
    if (length > 0) {
      WARNING (("Data supplied was invalid"));
    } else {
      WARNING (("Invalid file %s", data));
    }
    break;
    
  case BitmapNoMemory    :
    if (length > 0) {
      WARNING (("Not enough memory to load data"));
    } else {
      WARNING (("Not enough memory to load from file %s", data));
    }
    break;
    
  default :
    WARNING (("Never heard of this error %d", xbmError));
    break;
  }
}

/*************************************************************************
 * Based on the ReadBitmap routine of X11, but modifies heavily.
 *************************************************************************/
#if NeedFunctionPrototypes
int face_dataOfXbm(char *data, int length, Pixmap *image,
		   unsigned int *width, unsigned int *height)
#else
int face_dataOfXbm(data, length, image, width, height)
  char         *data;
  int           length;
  Pixmap       *image;
  unsigned int *width, *height;
#endif
{
  int    readHex, val, pixels, counter, ww, hh;
  char   word[256], *ptr, *result;
  Pixmap pix;

  readHex =  0;
  ww      =  0;
  hh      =  0;
  
  while ((length > 0) && (readHex == 0)) {
    /*
     * Read a word.
     */
    sscanf(data, "%s", word);
    length -= strlen(word) + 1;
    data   += strlen(word) + 1;
    ptr     = strchr(word, '_');

    if (ptr != NULL) {
      /*
       * Get the width of the icon.
       */
      if (strcmp(ptr, "_width") == 0)
	sscanf(data, " %d", &ww);

      /*
       * Get the height of the icon.
       */
      if (strcmp(ptr, "_height") == 0)
	sscanf(data, " %d", &hh);
    }

    if (strcmp(word, "short") == 0)
      return BitmapFileInvalid;

    /*
     * If a '{' was found then start reading the hex numbers.
     */
    if (strcmp(word, "{") == 0)
      readHex = 1;
  }

  if (!ww || !hh)
    return BitmapFileInvalid;

  pixels = (ww*hh + 7) / 8;
  result = XtMalloc(pixels);
  if (result == NULL)
    return BitmapNoMemory;

  /*
   * Read the real data.
   */
  counter = 0;
  while ((counter < pixels) && (readHex == 1)) {
    /*
     * See if there is still data.
     */
    if (length <= 0) {
      XtFree(result);
      return BitmapFileInvalid;
    }
      
    /*
     * See if it is a hex number.
     */
    if (strncmp(data, "0x", 2) == 0) {
      strcpy(word, "0x");
      length -= 2;
      data   += 2;
      val     = 0;
      while(length > 0) {
	if ((*data >= '0') && (*data <= '9'))
	  val = (val << 4) + (*data - '0');
	else if ((*data >= 'a') && (*data <= 'f'))
	  val = (val << 4) + (*data - 'a') + 10;
	else if ((*data >= 'A') && (*data <= 'F'))
	  val = (val << 4) + (*data - 'F') + 10;
	else
	  break;
	data++;
	length--;
      }
      result[counter++] = val & 0xFF;
    }

    /*
     * If a '}' was found then it is the end of the data.
     */
    if (*data == '}')
      readHex = 0;

    /*
     * Skip all the white spaces + ','.
     */
    while (((*data == '\n') || (*data == '\t') || (*data == ' ') ||
	    (*data == ',')) && (length > 0)) {
      data++;
      length--;
    }
  }

  /*
   * Return an error if there weren't enough pixels.
   */
  if (counter < pixels) {
    XtFree(result);
    return BitmapFileInvalid;
  }

  /*
   * Create a pixmap from the data.
   */
  pix = XCreateBitmapFromData(display, window, result, ww, hh);
  if (pix == None) {
    XtFree(result);
    return BitmapNoMemory;
  }
   
  /*
   * return the new pixmap.
   */
  *image  = pix;
  *width  = ww;
  *height = hh;
  XtFree(result);
  return BitmapSuccess;
}
