/*
 * $Id: facesdb_add.c,v 1.4 1995/05/29 15:10:13 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "facesdb.h"
#include <stdlib.h>

/*************************************************************************
 * 
 *************************************************************************/
#if NeedFunctionPrototypes
void addEntry(char *dbase, char *email, char *image, char *Major, char *Minor,
              long Length)
#else
void addEntry(dbase, email, image, Major, Minor, Length)
  char *dbase, *email, *image, *Major, *Minor;
  long  Length;
#endif
{
  FILE        *fp;
  GDBM_FILE    dbf;
  datum        key, content, entry;
  char         length[10];
  long         size;
  void        *ptr;
  Contents    *contents, *next;

  /*
   * Put the new stuff first in the entry.
   */
  sprintf(length, ":%ld:",  Length);
  size        = strlen(Major) + strlen(Minor) + strlen(length) + 1;
  entry.dsize = size + Length;
  if (Length == 0)
    entry.dsize += strlen(image) + 1;
  entry.dptr  = XtMalloc(entry.dsize);
  strcpy(entry.dptr, Major);
  strcat(entry.dptr, "/");
  strcat(entry.dptr, Minor);
  strcat(entry.dptr, length);

  if (Length ==  0) {
    strcat(entry.dptr, image);
    entry.dptr[entry.dsize] = 0;
  } else {
    /*
     * Read the data into the entry.
     */
    if ((fp = fopen(image, "rb")) == NULL) {
      if (!noTxt)
        fprintf(stderr, "Could not open image (%s).\n", image);
      return;
    }
    if (fread((entry.dptr + size), 1, Length, fp) != Length) {
      fclose(fp);
      if (!noTxt)
        fprintf(stderr, "Could not read the image (%s).\n", image);
      return;    
    }
    fclose(fp);
  }

  /*
   * Open the database or create it.
   */
  dbf = gdbm_open(dbase, block_size, GDBM_WRCREAT, mode, fatal_func);

  /*
   * Get the original info from the database.
   */
  key.dptr  = XtNewString(email);
  key.dsize = strlen(email);
  content   = gdbm_fetch(dbf, key);
  contents  = extractContents(&content);

  /*
   * Add all the original contents + the new content in the dbase.
   */
  while (contents != NULL) {
    if ((strcmp(contents->Major, Major) != 0) ||
	(strcmp(contents->Minor, Minor) != 0)) {
      /*
       * Add this to the entry.
       */
      sprintf(length, ":%d:",  contents->Length);
      size         = entry.dsize;
      entry.dsize += strlen(contents->Major) + strlen(contents->Minor) +
	             strlen(length) + contents->Length + 1;
      if (contents->Length == 0)
	entry.dsize += strlen(contents->Data) + 1;
      entry.dptr   = XtRealloc(entry.dptr, entry.dsize);
      ptr          = entry.dptr + size;
      strcpy(ptr, contents->Major);
      strcat(ptr, "/");
      strcat(ptr, contents->Minor);
      strcat(ptr, length);
      if (contents->Length > 0) {
	ptr = entry.dptr + size + strlen(Major) + strlen(Minor) +
	      strlen(length) + 1;
	memcpy(ptr, contents->Data, contents->Length);
      } else {
	strcat(ptr, contents->Data);
	entry.dptr[entry.dsize] = 0;
      }
    }
    next = contents->next;
    XtFree((char *)contents);
    contents = next;
  }

  /*
   * Add the entry to the database.
   */
  gdbm_store(dbf, key, entry, GDBM_REPLACE);

  /*
   * Close the database.
   */
  gdbm_close(dbf);

  /*
   * Free all the space allocated
   */
  XtFree(entry.dptr);
  XtFree(key.dptr);
  if (content.dptr != NULL)
    free(content.dptr);
}
