/*
 * $Id: facesdb_convert.c,v 1.5 1995/05/29 15:09:51 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "facesdb.h"
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/param.h>

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
char *nextToken(char *input)
#else
char *nextToken(input)
  char *input;
#endif
{
  char *src;

  src = input;
  /*
   * Find the first WS or the \0.
   */
  while ((*src != 0) && (*src != ' ') && (*src != '\t') && (*src != '\n'))
    src++;

  /*
   * Return NULL when \0 was found. Otherwise replace WS with \0.
   */
  if (*src == 0)
    return NULL;
  *src = 0; src++;

  /*
   * Move to the first letter after WS.
   */
  while ((*src != 0) && ((*src == ' ') || (*src == '\t') || (*src == '\n')))
    src++;

  if (*src == 0)
    return NULL;
  return src;
}

/*************************************************************************
 *
 *************************************************************************/
#if NeedFunctionPrototypes
void convertDatabase(char *faceDir)
#else
void convertDatabase(faceDir)
  char *faceDir;
#endif
{
  FILE        *fp;
  char         filename[MAXPATHLEN+1], image[MAXPATHLEN+1];
  char         line[200];
  char        *dir, *type, *ptr;
  char        *dbase, *email, *Major, *Minor;
  struct stat  buf;

  /*
   * Open the index file.
   */
  strcpy(filename, faceDir);
  strcat(filename, "/.index");
  if ((fp = fopen(filename, "r")) == NULL) {
    if (!noTxt)
      fprintf(stderr, "Could not open %s.\n", filename);
    exit(-1);
  }
  
  /*
   * Create the databasename
   */
  strcpy(filename, faceDir);
  strcat(filename, ".gdbm");
  dbase = XtNewString(filename);
  
  /*
   * Read all the entries. They are in the form:
   *    email dir image-type image-type .....
   */
  fgets(line, 80, fp);
  while (!feof(fp)) {
    email = line;
    dir   = nextToken(email);
    type  = nextToken(dir);
    while (type != NULL) {
      ptr = nextToken(type);
      sprintf(image, "%s/%s/face.%s", faceDir, dir, type);
      if (stat(image, &buf) < 0) {
	if (!noTxt)
	  fprintf(stderr, "Could not find %s.\n", image);
      } else {
	/*
	 * Store the info.
	 */
	lowerAddress(email);
	Minor = XtNewString(type);
	Major = XtNewString("image");
	  
	/*
	 * Add the face to the database
	 */
	if (!noTxt)
	  printf("-email %s -file %s -minor %s\n", email, image, Minor);
	addEntry(dbase, email, image, Major, Minor, buf.st_size);
	XtFree(email);
      }
      type = ptr;
    }
    fgets(line, 80, fp);
  }
  fclose(fp);
  XtFree(dbase);
}
