/*
 * $Id: facesdb_delete.c,v 1.3 1995/03/19 17:16:14 kooper Exp $
 *
 * Copyright (C) 1995, Rob Kooper (kooper@cc.gatech.edu)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "facesdb.h"
#include <stdlib.h>

/*************************************************************************
 * 
 *************************************************************************/
#if NeedFunctionPrototypes
void deleteEntry(char *dbase, char *address, char *Major, char *Minor)
#else
void deleteEntry(dbase, address, Major, Minor)
  char *dbase, *address, *Major, *Minor;
#endif
{
  GDBM_FILE    dbf;
  datum        key, content, entry;
  Contents    *contents, *next;
  char         length[256];
  void        *ptr;
  long         size;

  /*
   * Open the database or create it.
   */
  dbf = gdbm_open(dbase, block_size, GDBM_WRITER, mode, fatal_func);
  if (dbf == NULL) {
    if (!noTxt)
      fprintf(stderr, "Sorry, could not open database %s.\n", dbase);
    return;
  }

  /*
   * Create the key and get the info.
   */
  key.dptr  = XtNewString(address);
  key.dsize = strlen(address);
  content   = gdbm_fetch(dbf, key);
  if (content.dptr == NULL) {
    gdbm_close(dbf);
    return;
  }

  if (((Major == NULL) && (Minor == NULL)) || 
      (strncmp(content.dptr, "link:", 5) == 0)) {
    /*
     * Delete the person from the database
     */
    gdbm_delete(dbf, key);
  } else {    
    /*
     * Delete the info from the person
     */

    contents    = extractContents(&content);
    entry.dsize = 0;
    entry.dptr  = NULL;
    while (contents != NULL) {
      /*
       * Do we want to keep this piece of data?
       *   NULL/xpm   -> means delete all xpm     == keep all non xpm
       *   image/NULL -> means delete all images  == keep all non images
       *   image/xpm  -> means delete all xpm img == keep the rest
       */
      if (((Major == NULL) && (strcmp(Minor, contents->Minor) != 0)) ||
	  ((Minor == NULL) && (strcmp(Major, contents->Major) != 0)) ||
	  ((Minor != NULL) && (strcmp(Minor, contents->Minor) != 0)) ||
	  ((Major != NULL) && (strcmp(Major, contents->Major) != 0))) {
	/*
	 * Add this to the entry.
	 */
	sprintf(length, ":%d:",  contents->Length);
	size         = entry.dsize;
	entry.dsize += strlen(contents->Major) + strlen(contents->Minor) +
	               strlen(length) + contents->Length + 1;
	if (contents->Length == 0)
	  entry.dsize += strlen(contents->Data) + 1;
	entry.dptr   = XtRealloc(entry.dptr, entry.dsize);
	ptr          = entry.dptr + size;
	strcpy(ptr, contents->Major);
	strcat(ptr, "/");
	strcat(ptr, contents->Minor);
	strcat(ptr, length);
	if (contents->Length > 0)
	  strncat(ptr, contents->Data, contents->Length);
	else {
	  strcat(ptr, contents->Data);
	  entry.dptr[entry.dsize] = 0;
	}
      }
      next = contents->next;
      XtFree((char *)contents);
      contents = next;
    }

    /*
     * Add the entry to the database.
     */
    if (entry.dptr != NULL) {
      gdbm_store(dbf, key, entry, GDBM_REPLACE);
      XtFree(entry.dptr);
    } else
      gdbm_delete(dbf, key);
  }

  /*
   * Close the database.
   */
  gdbm_close(dbf);

  /*
   * Finish up.
   */
  free(content.dptr);
  XtFree(key.dptr);
}
